/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */

#include <string.h>

#ifndef _PTL3_P3_FORWARD_H_
#define _PTL3_P3_FORWARD_H_

/* Here we associate the index of each function in the library dispatch
 * table with the corresponding function name, which will be prefixed
 * with "lib_".   e.g. PTL_GETID is the index for the function do_PtlGetId.
 */
#define PTL_LIB_DISPATCH					\
PTL_DISPATCH_ENTRY( P3LIB_AC_ENTRY,	PtlACEntry),		\
PTL_DISPATCH_ENTRY( P3LIB_EQ_ALLOC,	PtlEQAlloc),		\
PTL_DISPATCH_ENTRY( P3LIB_EQ_FREE,	PtlEQFree),		\
PTL_DISPATCH_ENTRY( P3LIB_GET,		PtlGet),		\
PTL_DISPATCH_ENTRY( P3LIB_GET_JID,	PtlGetJid),		\
PTL_DISPATCH_ENTRY( P3LIB_GET_ID,	PtlGetId),		\
PTL_DISPATCH_ENTRY( P3LIB_GETPUT,	PtlGetPut),		\
PTL_DISPATCH_ENTRY( P3LIB_GET_UID,	PtlGetUid),		\
PTL_DISPATCH_ENTRY( P3LIB_MD_ATTACH,	PtlMDAttach),		\
PTL_DISPATCH_ENTRY( P3LIB_MD_BIND,	PtlMDBind),		\
PTL_DISPATCH_ENTRY( P3LIB_MD_UNLINK,	PtlMDUnlink),		\
PTL_DISPATCH_ENTRY( P3LIB_MD_UPDATE,	PtlMDUpdate),		\
PTL_DISPATCH_ENTRY( P3LIB_LIB_FINI,	PtlFini),		\
PTL_DISPATCH_ENTRY( P3LIB_LIB_INIT,	PtlInit),		\
PTL_DISPATCH_ENTRY( P3LIB_ME_ATTACH,	PtlMEAttach),		\
PTL_DISPATCH_ENTRY( P3LIB_ME_ATTACHANY,	PtlMEAttachAny),	\
PTL_DISPATCH_ENTRY( P3LIB_ME_DUMP,	PtlMEDump),		\
PTL_DISPATCH_ENTRY( P3LIB_ME_INSERT,	PtlMEInsert),		\
PTL_DISPATCH_ENTRY( P3LIB_ME_UNLINK,	PtlMEUnlink),		\
PTL_DISPATCH_ENTRY( P3LIB_NI_DEBUG,	PtlNIDebug),		\
PTL_DISPATCH_ENTRY( P3LIB_NI_FINI,	PtlNIFini),		\
PTL_DISPATCH_ENTRY( P3LIB_NI_INIT,	PtlNIInit),		\
PTL_DISPATCH_ENTRY( P3LIB_NI_DIST,	PtlNIDist),		\
PTL_DISPATCH_ENTRY( P3LIB_NI_STATUS,	PtlNIStatus),		\
PTL_DISPATCH_ENTRY( P3LIB_PROGRESS,	PtlProgress),		\
PTL_DISPATCH_ENTRY( P3LIB_PUT,		PtlPut),		\
PTL_DISPATCH_ENTRY( P3LIB_TBLDUMP,	PtlTblDump)


/* assign indices for functions in the dispatch table.
 * LIB_MAX_DISPATCH gives the number of entries in the dispatch table
 */
#define PTL_DISPATCH_ENTRY(fcn_id,fcn) fcn_id

enum { PTL_LIB_DISPATCH, LIB_MAX_DISPATCH };

#undef PTL_DISPATCH_ENTRY


struct p3_libreq {

	/* Index into the Portals library request dispatch function table
	 */
	unsigned call_index;

	/* Most request functions need to reference at least one library
	 * object.
	 */
	ptl_handle_any_t handle;

	/* Many request functions need to supply an integer value; a 
	 * ptl_size_t can hold any such value without losing information.
	 */
	ptl_size_t value;

	union {					/* variable length, so last */
		struct {
			ptl_pt_index_t pt_index;
			ptl_process_id_t match_id;
			ptl_uid_t user_id;
			ptl_jid_t job_id;
		} ace;

		struct {
			void *base;
			ptl_size_t len;
		} eqalloc;

		struct {
			ptl_handle_md_t put_md;
			ptl_process_id_t target;
			ptl_pt_index_t pt_idx;
			ptl_ac_index_t ac_idx;
			ptl_size_t rem_os;
			ptl_hdr_data_t hdr_data;
		} getput;

		struct {
			ptl_md_t md;
			ptl_md_iovec_t iov[1];	/* variable length, so last */
		} md;

		struct {
			ptl_handle_eq_t testq;
			ptl_md_t new_md;
			int new_md_valid;
			int old_md_valid;
			ptl_md_iovec_t iov[1];	/* variable length, so last */
		} mdupdate;

		struct {
			ptl_pt_index_t pt_index;
			ptl_process_id_t match_id;
			ptl_match_bits_t ibits;
			ptl_ins_pos_t position;
			ptl_unlink_t unlink;
		} me;

		struct {
			ptl_size_t loc_os;
			ptl_size_t loc_len;
			ptl_ack_req_t ack_req;
			ptl_process_id_t target;
			ptl_pt_index_t pt_idx;
			ptl_ac_index_t ac_idx;
			ptl_size_t rem_os;
			ptl_hdr_data_t hdr_data;
			int region;
		} move;

		struct {
			ptl_interface_t type;
			ptl_ni_limits_t limits;
			size_t data_sz;
			unsigned long data[1];	/* variable length, so last */
		} niinit;
	} t;
};

typedef struct p3_libreq p3_libreq_t;

#define PTL_LIBREQ_INIT {LIB_MAX_DISPATCH, PTL_INVALID_HANDLE, }
#define ptl_init_libreq(req)	\
do {				\
	memset(req, 0, sizeof(p3_libreq_t));	\
	req->call_index = LIB_MAX_DISPATCH;	\
	req->handle = PTL_INVALID_HANDLE;	\
} while (0)


struct p3_libres {

	int status;

	/* Many request functions will generate a result handle.
	 */
	ptl_handle_any_t handle;

	/* Some request functions will return an integer value; a 
	 * ptl_size_t can hold any such value without losing information.
	 */
	ptl_size_t value;

	union {
		struct {
			ptl_process_id_t id;
		} getid;

		struct {
			ptl_md_t old_md;
			ptl_md_iovec_t iov[1];	/* variable length, so last */
		} mdupdate;

		struct {
			ptl_ni_limits_t limits;
		} niinit;
	} t;
};

typedef struct p3_libres p3_libres_t;

#define PTL_LIBRES_INIT {PTL_FAIL, PTL_INVALID_HANDLE, }
#define ptl_init_libres(res)	\
do {				\
	memset(res, 0, sizeof(p3_libres_t));	\
	res->status = PTL_FAIL;			\
	res->handle = PTL_INVALID_HANDLE;	\
} while (0)

typedef struct p3_forward {
	void *request;
	void *result;
	size_t req_len;
	size_t res_len;
} p3_forward_t;


/* Call this to initialize communications with the Portals3 library.
 * Returns 0 on success.
 */
int p3_libstart(void);

/* Call this to shut down communications with the Portals3 library.
 * Returns 0 on success.
 */
void p3_libstop(void);

/* Call this to forward a request to the Portals3 library.  Since some
 * of the requests and replies contain an iovec, they can be varible-sized,
 * and we need a length for each.
 */
void p3_forward(void *request, size_t reqlen, void *result, size_t reslen);

/* Call this to terminate request forwarding.
 */
void p3_forward_shutdown(void);

#endif /* _PTL3_P3_FORWARD_H_ */
