/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */

/* OK, this probably qualifies as code abuse, but I hate cut'n'paste.
 *
 * I want a macro to build the wrapper functions to make system calls from
 * kernel code.  Thus I need to generate a function that calls another function
 * with exactly the same arguments.
 *
 * Due to limitations in macro processing, I can't give a macro a list
 * of typed arguments (e.g. "int i") and have one part of the macro use the
 * typed arguments, and another just the variable names.
 *
 * I can't use variable-length argument list macros, because I need _two_ 
 * variable-length arguments to the macro, one for the parameter list and
 * one for the calling argument list.  So, I need a different separator that
 * I can redefine to be a comma.
 *
 * I use "_" since gnu cpp, at least, thinks that's a valid macro name.  As
 * long as any "_" intended to be transformed into a comma is delimited by
 * white space, cpp recognizes it as a separate token.  Thus, you can still
 * use type names containing underscores.
 *
 * So, when you use wrap_sys_call(), list_args should be the parameter list
 * of the call, with "_" replacing "," between parameters.  call_args
 * should be the variable list used to make the call, with "_" replacing ","
 * between variables.
 */

#ifndef _PTL3_KERN_SYSCALL_H_
#define _PTL3_KERN_SYSCALL_H_

#include <asm/uaccess.h>
#include <linux/syscalls.h>
#include <kern_errno.h>

typedef int socklen_t;

extern int p3_dequeue_signal(int err);

#ifdef NEED_P3_DEQUEUE_SIGNAL
#define check_signal(rtn) do {rtn = p3_dequeue_signal(rtn);} while (0)
#else
#define check_signal(rtn) do {} while (0)
#endif

#define wrap_sys_call(rtn_t,call,sys_call,list_args,call_args)	\
static inline							\
rtn_t call(list_args)						\
{								\
        int rtn;						\
        mm_segment_t old_fs;					\
        old_fs = get_fs();					\
        set_fs(KERNEL_DS);					\
        rtn = sys_call(call_args);				\
        set_fs(old_fs);						\
	check_signal(rtn);					\
	if (rtn < 0)						\
		kern_errno = -rtn;				\
        return rtn;						\
}

#define _ ,

wrap_sys_call(long, open, sys_open,
	      const char __user *filename _ int flags _ int mode,
	      filename _ flags _ mode)

wrap_sys_call(long, close, sys_close,
	      unsigned int fd,
	      fd)

wrap_sys_call(long, ioctl, sys_ioctl,
	      unsigned int fd _ unsigned int cmd _ unsigned long arg,
	      fd _ cmd _ arg)

wrap_sys_call(ssize_t, read, sys_read,
	      unsigned int fd _ char __user *buf _ size_t count,
	      fd _ buf _ count)

wrap_sys_call(ssize_t, write, sys_write,
	      unsigned int fd _ const char __user *buf _ size_t count,
	      fd _ buf _ count)

wrap_sys_call(long, fstat, sys_newfstat,
	      unsigned int fd _ struct stat __user *statbuf,
	      fd _ statbuf)

wrap_sys_call(long, stat, sys_newstat,
	      char *filename _ struct stat __user *statbuf,
	      filename _ statbuf)

wrap_sys_call(long, lstat, sys_newlstat,
	      char __user *filename _ struct stat __user *statbuf,
	      filename _ statbuf)

wrap_sys_call(long, fcntl, sys_fcntl,
	      unsigned int fd _ unsigned int cmd _ unsigned long arg,
	      fd _ cmd _ arg)

wrap_sys_call(long, socket, sys_socket,
	      int family _ int type _ int protocol,
	      family _ type _ protocol)

wrap_sys_call(long, bind, sys_bind,
	      int fd _ struct sockaddr __user *umyaddr _ int addrlen,
	      fd _ umyaddr _ addrlen)

wrap_sys_call(long, connect, sys_connect,
	      int fd _ struct sockaddr __user *useraddr _ int addrlen,
	      fd _ useraddr _ addrlen)

wrap_sys_call(long, listen, sys_listen,
	      int fd _ int backlog,
	      fd _ backlog)

wrap_sys_call(long, accept, sys_accept,
	      int fd _ struct sockaddr __user *sockaddr _ int __user *addrlen,
	      fd _ sockaddr _ addrlen)

wrap_sys_call(long, getsockname, sys_getsockname,
	      int fd _ struct sockaddr __user *sockaddr _ int __user *addrlen,
	      fd _ sockaddr _ addrlen)

wrap_sys_call(long, getpeername, sys_getpeername,
	      int fd _ struct sockaddr __user *sockaddr _ int __user *addrlen,
	      fd _ sockaddr _ addrlen)

wrap_sys_call(long, socketpair, sys_socketpair,
	      int family _ int type _ int protocol _ int __user sockvec[2],
	      family _ type _ protocol _ sockvec)

wrap_sys_call(long, send, sys_send,
	      int fd _ void __user *buf _ size_t len _ unsigned flags,
	      fd _ buf _ len _ flags)

wrap_sys_call(long, recv, sys_recv,
	      int fd _ void  __user *ubuf _ size_t size _ unsigned flags,
	      fd _ ubuf _ size _ flags)

wrap_sys_call(long, shutdown, sys_shutdown,
	      int fd _ int how,
	      fd _ how)

wrap_sys_call(long,setsockopt, sys_setsockopt,
	      int fd _ int level _ int optname _
	      char __user  *optval _ int optlen,
	      fd _ level _ optname _ optval _ optlen)

wrap_sys_call(long, getsockopt, sys_getsockopt,
	      int fd _ int level _ int optname _
	      char __user  *optval _ int __user  *optlen,
	      fd _ level _ optname _ optval _ optlen)

wrap_sys_call(long, select, sys_select,
	      int n _ fd_set __user  *inp _ fd_set __user *outp _ 
	      fd_set __user  *exp _ struct timeval __user *tvp,
	      n _ inp _ outp _ exp _ tvp)

wrap_sys_call(ssize_t,readv, sys_readv,
	      unsigned long fd _
	      const struct iovec __user *vector _ unsigned long count,
	      fd _ vector _ count)

wrap_sys_call(ssize_t, writev, sys_writev,
	      unsigned long fd _
	      const struct iovec __user *vector _ unsigned long count,
	      fd _ vector _ count)

#undef _
#undef wrap_syscall

#endif /* _PTL3_KERN_SYSCALL_H_ */
