/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */

#include <linux/spinlock.h>
#include <linux/interrupt.h>

#ifndef _PTL3_KERN_P3LOCK_H_
#define _PTL3_KERN_P3LOCK_H_

typedef spinlock_t _p3lock;

#define _P3LOCK_INIT_UNLOCKED SPIN_LOCK_UNLOCKED

#define _p3lock_init(lock) spin_lock_init(lock)
#define _p3lock_destroy(lock) do {} while (0)

/* With the TCP or UDP NALs, all we need for locking is spin_lock/spin_unlock.
 * For debugging, spin_lock_bh/spin_unlock_bh is nice because it's a little
 * more restrictive lock, but it increases latency a little on SMP machines.
 *
 * For NALs implemented with tasklets, spin_lock_bh/spin_unlock_bh would be
 * required.
 *
 * For NALs implemented with an interrupt handler, we probably couldn't
 * get away with one type of lock, because functions that could be called
 * either from interrupt context or not would need to save flags.
 *
 * But we'll save that for another day.  Such NALs probably wouldn't also
 * be optionally compiled for user-space operation, and could probably
 * use kernel functions directly, without wrapping.
 */
#if defined P3LOCK_SPINLOCK
#define _p3_lock(lock) spin_lock(lock)
#define _p3_unlock(lock) spin_unlock(lock)
#elif defined P3LOCK_SPINLOCK_BH
#define _p3_lock(lock) spin_lock_bh(lock)
#define _p3_unlock(lock) spin_unlock_bh(lock)
#endif

#endif /* _PTL3_KERN_P3LOCK_H_ */
