/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */

#include <stdint.h>
#include <linux/module.h>
#include <linux/in.h>
#include <kern_errno.h>

#include "p3utils.h"

int kern_errno;
char kern_error_str[KERN_ERROR_BUF_LEN];
EXPORT_SYMBOL_GPL(kern_errno);
EXPORT_SYMBOL_GPL(kern_error_str);

#ifdef P3_PRINT_SHOWS_CURRENT
char p3_print_buf[NR_CPUS][P3_PRINT_BUFSZ];
EXPORT_SYMBOL_GPL(p3_print_buf);
spinlock_t p3_print_lock = SPIN_LOCK_UNLOCKED;
EXPORT_SYMBOL_GPL(p3_print_lock);
#endif


void *bsearch(const void *key, const void *base, size_t nmemb, size_t size,
	      int (*compare)(const void*, const void*))
{
	size_t f = 0, l = nmemb-1;

	if (!(nmemb && key && base && compare))
		return NULL;

	while (f < l) {
		size_t m = (f+l) >> 1;

		if (compare(key, (char *)base + m*size) > 0)
			f = m+1;
		else
			l = m;
	}
	if (compare(key, (char *)base + f*size) == 0)
		return (char *)base + f*size;

	return NULL;
}
EXPORT_SYMBOL_GPL(bsearch);


char inet_ntoa_buf[INET_NTOA_BUF_SZ];

char *inet_ntoa(struct in_addr in)
{
	uint32_t a = ntohl(in.s_addr);
	snprintf(inet_ntoa_buf, INET_NTOA_BUF_SZ, "%d.%d.%d.%d",
		 a >> 3*CHAR_BIT & 0xff,
		 a >> 2*CHAR_BIT & 0xff,
		 a >>   CHAR_BIT & 0xff,
		 a               & 0xff);
	return inet_ntoa_buf;
}
EXPORT_SYMBOL_GPL(inet_ntoa);
