/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */

#include <sys/types.h>

/* If compiling for user-space (and maybe NIC-space), these need to be
 * the Portals3 versions of the Linux kernel include files, which
 * have been suitably modified for use in user-space code.
 */
#include <linux/list.h>

/* These are all Portals3 include files.
 */
#include <p3-config.h>
#include <p3utils.h>

#include <p3/lock.h>

#include <p3api/types.h>
#include <p3api/api.h>
#include <p3api/nal.h>

#include <p3/handle.h>
#include <p3/process.h>
#include <p3/forward.h>
#include <p3/errno.h>

#include "init.h"


p3_process_t p3_api_process = {-1, PTL_JID_ANY, };


int PtlInit(int *max_interfaces)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (p3_api_process.init >= 0)
		return PTL_OK;

	if (!max_interfaces)
		return PTL_SEGV;

	p3utils_init();

	req.call_index = P3LIB_LIB_INIT;
	p3_forward(&req, sizeof(req), &res, sizeof(res));

	if (res.status == PTL_OK) {
		p3_api_process.init = 0;
		*max_interfaces = PTL_MAX_INTERFACES;
	}

	return res.status;
}

void PtlFini(void)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	unsigned i;

	if (p3_api_process.init < 0)
		return;

	for (i=0; i<PTL_MAX_INTERFACES; i++) {
		if (p3_api_process.ni[i]) {
			api_ni_t *ni = p3_api_process.ni[i];
			ptl_handle_ni_t handle = PTL_OBJ_HNDL(ni);
			PtlNIFini(handle);
		}
	}
	req.call_index = P3LIB_LIB_FINI;
	p3_forward(&req, sizeof(req), &res, sizeof(res));

	p3_api_process.init = -1;
	p3_api_process.jid = PTL_JID_ANY;

	p3_forward_shutdown();
}
