/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */

#include <sys/types.h>

/* If compiling for user-space (and maybe NIC-space), these need to be
 * the Portals3 versions of the Linux kernel include files, which
 * have been suitably modified for use in user-space code.
 */
#include <linux/list.h>

/* These are all Portals3 include files.
 */
#include <p3-config.h>
#include <p3utils.h>

#include <p3/lock.h>

#include <p3api/types.h>
#include <p3api/nal.h>
#include <p3api/api.h>
#include <p3api/debug.h>

#include <p3/handle.h>
#include <p3/process.h>
#include <p3/forward.h>
#include <p3/errno.h>
#include <p3/debug.h>

#include "init.h"


int PtlMDAttach(ptl_handle_me_t me_handle,
		ptl_md_t md,
		ptl_unlink_t unlink_op,
		ptl_handle_md_t *md_handle)
{
	p3_libreq_t *req;
	p3_libres_t res = PTL_LIBRES_INIT;
	unsigned if_idx = PTL_NI_INDEX(me_handle);
	int req_len = sizeof(*req);

	if (!md_handle)
		return PTL_SEGV;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (PTL_NI_INDEX(md.eq_handle) != if_idx)
		return PTL_MD_ILLEGAL;

	if (!(if_idx < PTL_MAX_INTERFACES && p3_api_process.ni[if_idx]))
		return PTL_ME_INVALID;

	if (md.options & PTL_MD_IOVEC) {
		req_len += md.length * sizeof(ptl_md_iovec_t);
	}
	if (!(req = p3_malloc(req_len)))
		return PTL_NO_SPACE;

	ptl_init_libreq(req);

	req->call_index = P3LIB_MD_ATTACH;
	req->handle = me_handle;
	req->value = unlink_op;
	req->t.md.md = md;

	if (md.options & PTL_MD_IOVEC) {
		ptl_size_t i;
		ptl_md_iovec_t *iov = md.start;
		for (i=0; i<md.length; i++) {
			req->t.md.iov[i].iov_base = iov[i].iov_base;
			req->t.md.iov[i].iov_len  = iov[i].iov_len;
		}
	}
	else {
		req->t.md.iov[0].iov_base = md.start;
		req->t.md.iov[0].iov_len = md.length;
	}
	p3_forward(req, req_len, &res, sizeof(res));
	*md_handle = res.handle;

	p3_free(req);
	return res.status;
}

int PtlMDBind(ptl_handle_ni_t ni_handle,
	      ptl_md_t md,
	      ptl_unlink_t unlink_op,
	      ptl_handle_md_t *md_handle)
{
	p3_libreq_t *req;
	p3_libres_t res = PTL_LIBRES_INIT;
	unsigned if_idx = PTL_NI_INDEX(ni_handle);
	int req_len = sizeof(*req);

	if (!md_handle)
		return PTL_SEGV;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (PTL_NI_INDEX(md.eq_handle) != if_idx)
		return PTL_MD_ILLEGAL;

	if (!(if_idx < PTL_MAX_INTERFACES && p3_api_process.ni[if_idx]))
		return PTL_NI_INVALID;

	if (md.options & PTL_MD_IOVEC) {
		req_len += md.length * sizeof(ptl_md_iovec_t);
	}
	if (!(req = p3_malloc(req_len)))
		return PTL_NO_SPACE;

	ptl_init_libreq(req);

	req->call_index = P3LIB_MD_BIND;
	req->handle = ni_handle;
	req->value = unlink_op;
	req->t.md.md = md;

	if (md.options & PTL_MD_IOVEC) {
		ptl_size_t i;
		ptl_md_iovec_t *iov = md.start;
		for (i=0; i<md.length; i++) {
			req->t.md.iov[i].iov_base = iov[i].iov_base;
			req->t.md.iov[i].iov_len  = iov[i].iov_len;
		}
	}
	else {
		req->t.md.iov[0].iov_base = md.start;
		req->t.md.iov[0].iov_len = md.length;
	}
	p3_forward(req, req_len, &res, sizeof(res));
	*md_handle = res.handle;

	p3_free(req);
	return res.status;
}

int PtlMDUnlink(ptl_handle_md_t md_handle)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(md_handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(md_handle)]))
		return PTL_MD_INVALID;

	req.call_index = P3LIB_MD_UNLINK;
	req.handle = md_handle;

	p3_forward(&req, sizeof(req), &res, sizeof(res));
	return res.status;
}

/* Note: if the user expects that the MD to be copied into *old_md will
 * contain an iovec, then old_md->start needs to point to enough memory
 * to hold the largest expected iovec, and old_md->length needs to be
 * how long the largest expected iovec can be, and old_md->options must
 * have PTL_MD_IOVEC set.
 */
int PtlMDUpdate(ptl_handle_md_t md_handle,
		ptl_md_t *old_md,
		ptl_md_t *new_md,
		ptl_handle_eq_t eq_handle)
{
	p3_libreq_t *req;
	p3_libres_t *res;
	api_ni_t *ni;
	api_eq_t *eq;
	ptl_seq_t sequence = -1;
	int req_len = sizeof(*req);
	int res_len = sizeof(*res);
	int rc;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(md_handle) < PTL_MAX_INTERFACES &&
	      (ni = p3_api_process.ni[PTL_NI_INDEX(md_handle)])))
		return PTL_MD_INVALID;

	if (eq_handle != PTL_EQ_NONE) {
		if (!(VALID_PTL_OBJ(&ni->eq, eq_handle) &&
		      TST_OBJ(eq = GET_PTL_OBJ(&ni->eq,eq_handle),OBJ_INUSE)))
			return PTL_EQ_INVALID;
		sequence = eq->sequence;
	}
	if (new_md && new_md->options & PTL_MD_IOVEC)
		req_len += new_md->length * sizeof(ptl_md_iovec_t);

	if (!(req = p3_malloc(req_len)))
		return PTL_NO_SPACE;

	ptl_init_libreq(req);

	if (old_md && old_md->options & PTL_MD_IOVEC)
		res_len += old_md->length * sizeof(ptl_md_iovec_t);

	if (!(res = p3_malloc(res_len))) {
		p3_free(req);
		return PTL_NO_SPACE;
	}
	ptl_init_libres(res);

	req->call_index = P3LIB_MD_UPDATE;
	req->handle = md_handle;
	req->value = sequence;

	req->t.mdupdate.testq = eq_handle;

	if (old_md) {
		req->t.mdupdate.old_md_valid = 1;

		if (old_md->options & PTL_MD_IOVEC) {
			res->t.mdupdate.old_md.options |= PTL_MD_IOVEC;
			res->t.mdupdate.old_md.length = old_md->length;
		}
	}
	else
		req->t.mdupdate.old_md_valid = 0;

	if (new_md) {
		req->t.mdupdate.new_md_valid = 1;
		req->t.mdupdate.new_md = *new_md;

		if (new_md->options & PTL_MD_IOVEC) {
			ptl_size_t i;
			ptl_md_iovec_t *iov = new_md->start;
			for (i=0; i<new_md->length; i++) {

				req->t.mdupdate.iov[i].iov_base
					= iov[i].iov_base;

				req->t.mdupdate.iov[i].iov_len
					= iov[i].iov_len;
			}
		}
		else {
			req->t.mdupdate.iov[0].iov_base = new_md->start;
			req->t.mdupdate.iov[0].iov_len = new_md->length;
		}
	}
	p3_forward(req, req_len, res, res_len);
	rc = res->status;

	if (rc != PTL_OK)
		goto out;

	if (old_md) {
		ptl_md_t *lib_md = &res->t.mdupdate.old_md;
		ptl_md_iovec_t *old_iov = old_md->start;

		*old_md = *lib_md;

		if (lib_md->options & PTL_MD_IOVEC) {
			ptl_md_iovec_t *lib_iov = res->t.mdupdate.iov;
			int i = lib_md->length;

			while (i--) {
				old_iov[i].iov_base = lib_iov[i].iov_base;
				old_iov[i].iov_len  = lib_iov[i].iov_len;
			}
		}
	}
out:
	p3_free(req);
	p3_free(res);
	return rc;
}
