/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */

#include <sys/types.h>

/* If compiling for user-space (and maybe NIC-space), these need to be
 * the Portals3 versions of the Linux kernel include files, which
 * have been suitably modified for use in user-space code.
 */
#include <linux/list.h>

/* These are all Portals3 include files.
 */
#include <p3-config.h>
#include <p3utils.h>

#include <p3/lock.h>

#include <p3api/types.h>
#include <p3api/nal.h>
#include <p3api/api.h>
#include <p3api/debug.h>

#include <p3/handle.h>
#include <p3/process.h>
#include <p3/forward.h>
#include <p3/errno.h>
#include <p3/debug.h>

#include "init.h"


int PtlMEAttach(ptl_handle_ni_t ni_handle,
		ptl_pt_index_t pt_index,
		ptl_process_id_t match_id,
		ptl_match_bits_t match_bits,
		ptl_match_bits_t ignore_bits,
		ptl_unlink_t unlink,
		ptl_ins_pos_t position,
		ptl_handle_me_t *me_handle)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (!me_handle)
		return PTL_SEGV;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(ni_handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(ni_handle)]))
		return PTL_NI_INVALID;

	req.call_index = P3LIB_ME_ATTACH;
	req.handle = ni_handle;
	req.value = match_bits;

	req.t.me.pt_index = pt_index;
	req.t.me.match_id = match_id;
	req.t.me.ibits = ignore_bits;
	req.t.me.position = position;
	req.t.me.unlink = unlink;

	p3_forward(&req, sizeof(req), &res, sizeof(res));

	*me_handle = res.handle;

	return res.status;
}

int PtlMEAttachAny(ptl_handle_ni_t ni_handle,
		   ptl_pt_index_t *pt_index,
		   ptl_process_id_t match_id,
		   ptl_match_bits_t match_bits,
		   ptl_match_bits_t ignore_bits,
		   ptl_unlink_t unlink,
		   ptl_handle_me_t *me_handle)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (!(me_handle && pt_index))
		return PTL_SEGV;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(ni_handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(ni_handle)]))
		return PTL_NI_INVALID;

	req.call_index = P3LIB_ME_ATTACHANY;
	req.handle = ni_handle;
	req.value = match_bits;

	req.t.me.match_id = match_id;
	req.t.me.ibits = ignore_bits;
	req.t.me.unlink = unlink;

	p3_forward(&req, sizeof(req), &res, sizeof(res));

	*me_handle = res.handle;
	*pt_index = res.value;

	return res.status;
}

int PtlMEInsert(ptl_handle_me_t base,
		ptl_process_id_t match_id,
		ptl_match_bits_t match_bits,
		ptl_match_bits_t ignore_bits,
		ptl_unlink_t unlink,
		ptl_ins_pos_t position,
		ptl_handle_me_t *me_handle)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (!me_handle)
		return PTL_SEGV;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(base) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(base)]))
		return PTL_ME_INVALID;

	req.call_index = P3LIB_ME_INSERT;
	req.handle = base;
	req.value = match_bits;

	req.t.me.match_id = match_id;
	req.t.me.ibits = ignore_bits;
	req.t.me.position = position;
	req.t.me.unlink = unlink;

	p3_forward(&req, sizeof(req), &res, sizeof(res));

	*me_handle = res.handle;

	return res.status;
}

int PtlMEUnlink(ptl_handle_me_t me_handle)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(me_handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(me_handle)]))
		return PTL_ME_INVALID;

	req.call_index = P3LIB_ME_UNLINK;
	req.handle = me_handle;

	p3_forward(&req, sizeof(req), &res, sizeof(res));

	return res.status;
}

/* This function isn't part of the spec. See p3api/debug.h for an explanation.
 */
int PtlMEDump(ptl_handle_me_t me_handle)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(me_handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(me_handle)]))
		return PTL_ME_INVALID;

	req.call_index = P3LIB_ME_DUMP;
	req.handle = me_handle;

	p3_forward(&req, sizeof(req), &res, sizeof(res));

	return res.status;
}
