/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */

#include <sys/types.h>

/* If compiling for user-space (and maybe NIC-space), these need to be
 * the Portals3 versions of the Linux kernel include files, which
 * have been suitably modified for use in user-space code.
 */
#include <linux/list.h>

/* These are all Portals3 include files.
 */
#include <p3-config.h>
#include <p3utils.h>

#include <p3/lock.h>

#include <p3api/types.h>
#include <p3api/nal.h>
#include <p3api/api.h>
#include <p3api/debug.h>

#include <p3/handle.h>
#include <p3/process.h>
#include <p3/forward.h>
#include <p3/errno.h>
#include <p3/debug.h>

#include "init.h"

int PtlPut(ptl_handle_md_t md_handle,
	   ptl_ack_req_t ack_req,
	   ptl_process_id_t target_id,
	   ptl_pt_index_t pt_index,
	   ptl_ac_index_t ac_index,
	   ptl_match_bits_t match_bits,
	   ptl_size_t remote_offset,
	   ptl_hdr_data_t hdr_data)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(md_handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(md_handle)]))
		return PTL_MD_INVALID;

	req.call_index = P3LIB_PUT;
	req.handle = md_handle;
	req.value = match_bits;

	req.t.move.region = 0;
	req.t.move.loc_os = 0;
	req.t.move.loc_len = (ptl_size_t)-1;
	req.t.move.ack_req = ack_req;
	req.t.move.target = target_id;
	req.t.move.pt_idx = pt_index;
	req.t.move.ac_idx = ac_index;
	req.t.move.rem_os = remote_offset;
	req.t.move.hdr_data = hdr_data;

	p3_forward(&req, sizeof(req), &res, sizeof(res));
	return res.status;
}

int PtlPutRegion(ptl_handle_md_t md_handle,
		 ptl_size_t local_offset,
		 ptl_size_t length,
		 ptl_ack_req_t ack_req,
		 ptl_process_id_t target_id,
		 ptl_pt_index_t pt_index,
		 ptl_ac_index_t ac_index,
		 ptl_match_bits_t match_bits,
		 ptl_size_t remote_offset,
		 ptl_hdr_data_t hdr_data)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(md_handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(md_handle)]))
		return PTL_MD_INVALID;

	req.call_index = P3LIB_PUT;
	req.handle = md_handle;
	req.value = match_bits;

	req.t.move.region = 1;
	req.t.move.loc_os = local_offset;
	req.t.move.loc_len = length;
	req.t.move.ack_req = ack_req;
	req.t.move.target = target_id;
	req.t.move.pt_idx = pt_index;
	req.t.move.ac_idx = ac_index;
	req.t.move.rem_os = remote_offset;
	req.t.move.hdr_data = hdr_data;

	p3_forward(&req, sizeof(req), &res, sizeof(res));
	return res.status;
}

int PtlGet(ptl_handle_md_t md_handle,
	   ptl_process_id_t target_id,
	   ptl_pt_index_t pt_index,
	   ptl_ac_index_t ac_index,
	   ptl_match_bits_t match_bits,
	   ptl_size_t remote_offset)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(md_handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(md_handle)]))
		return PTL_MD_INVALID;

	req.call_index = P3LIB_GET;
	req.handle = md_handle;
	req.value = match_bits;

	req.t.move.region = 0;
	req.t.move.loc_os = 0;
	req.t.move.loc_len = (ptl_size_t)-1;
	req.t.move.target = target_id;
	req.t.move.pt_idx = pt_index;
	req.t.move.ac_idx = ac_index;
	req.t.move.rem_os = remote_offset;

	p3_forward(&req, sizeof(req), &res, sizeof(res));
	return res.status;
}

int PtlGetRegion(ptl_handle_md_t md_handle,
		 ptl_size_t local_offset,
		 ptl_size_t length,
		 ptl_process_id_t target_id,
		 ptl_pt_index_t pt_index,
		 ptl_ac_index_t ac_index,
		 ptl_match_bits_t match_bits,
		 ptl_size_t remote_offset)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(md_handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(md_handle)]))
		return PTL_MD_INVALID;

	req.call_index = P3LIB_GET;
	req.handle = md_handle;
	req.value = match_bits;

	req.t.move.region = 1;
	req.t.move.loc_os = local_offset;
	req.t.move.loc_len = length;
	req.t.move.target = target_id;
	req.t.move.pt_idx = pt_index;
	req.t.move.ac_idx = ac_index;
	req.t.move.rem_os = remote_offset;

	p3_forward(&req, sizeof(req), &res, sizeof(res));
	return res.status;
}

int PtlGetPut(ptl_handle_md_t get_md_handle, 
	      ptl_handle_md_t put_md_handle,
	      ptl_process_id_t target_id, 
	      ptl_pt_index_t pt_index,
	      ptl_ac_index_t ac_index, 
	      ptl_match_bits_t match_bits,
	      ptl_size_t remote_offset, 
	      ptl_hdr_data_t hdr_data)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(get_md_handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(get_md_handle)]))
		return PTL_MD_INVALID;

	if (PTL_NI_INDEX(put_md_handle) != PTL_NI_INDEX(get_md_handle))
		return PTL_MD_ILLEGAL;

	req.call_index = P3LIB_GETPUT;
	req.handle = get_md_handle;
	req.value = match_bits;

	req.t.getput.put_md = put_md_handle;
	req.t.getput.target = target_id;
	req.t.getput.pt_idx = pt_index;
	req.t.getput.ac_idx = ac_index;
	req.t.getput.rem_os = remote_offset;
	req.t.getput.hdr_data = hdr_data;

	p3_forward(&req, sizeof(req), &res, sizeof(res));
	return res.status;
}

/*
 * This function isn't part of the spec, but it should be.  An implementation
 * that has no progress thread anywhere (i.e. there is no NIC, kernel, or
 * user-space thread that can independently make progress) needs this
 * function to force progress to be made on outstanding messages.
 *
 * Anyway, we use it internally, but a library user needs to know it is
 * here to access it.  Also, a library user would only need it if not
 * using events at all.
 */
int PtlProgress(ptl_handle_any_t handle)
{
	p3_libreq_t req = PTL_LIBREQ_INIT;
	p3_libres_t res = PTL_LIBRES_INIT;

	if (p3_api_process.init < 0)
		return PTL_NO_INIT;

	if (!(PTL_NI_INDEX(handle) < PTL_MAX_INTERFACES &&
	      p3_api_process.ni[PTL_NI_INDEX(handle)]))
		return PTL_NI_INVALID;

	req.call_index = P3LIB_PROGRESS;
	req.handle = handle;

	p3_forward(&req, sizeof(req), &res, sizeof(res));
	return res.status;
}
