/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */

#include <p3-config.h>

#ifndef PTL_KERNEL_BLD
#include <sys/types.h>
#endif
#include <limits.h>

/* If compiling for user-space (and maybe NIC-space), these need to be
 * the Portals3 versions of the Linux kernel include files, which
 * have been suitably modified for use in user-space code.
 */
#include <linux/list.h>

/* These are all Portals3 include files.
 */
#include <p3api/types.h>

#include <p3/lock.h>
#include <p3/handle.h>
#include <p3/process.h>
#include <p3/forward.h>
#include <p3/errno.h>

#include <p3lib/types.h>
#include <p3lib/p3lib.h>
#include <p3lib/nal.h>
#include <p3lib/dispatch.h>

void lib_PtlACEntry(lib_ni_t *ni, p3_libreq_t *req, p3_libres_t *res)
{
        lib_ace_t *ace;
	ptl_ac_index_t ac_idx = req->value;

	if (ac_idx >= ni->actab.size) {
		res->status = PTL_AC_INDEX_INVALID;
		return;
	}
	if (req->t.ace.pt_index >= ni->ptltab.size &&
	    req->t.ace.pt_index != PTL_PT_INDEX_ANY) {
		res->status = PTL_PT_INDEX_INVALID;
		return;
	}
	ace = ni->actab.ace + ac_idx;

	p3_lock(&ni->obj_update);
	ace->id = req->t.ace.match_id;
	ace->uid = req->t.ace.user_id;
	ace->jid = req->t.ace.job_id;
	ace->ptl = req->t.ace.pt_index;
	p3_unlock(&ni->obj_update);

	res->status = PTL_OK;
}
