/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */
#include <p3-config.h>

#include <unistd.h>
#include <limits.h>

/* If compiling for user-space (and maybe NIC-space), these need to be
 * the Portals3 versions of the Linux kernel include files, which
 * have been suitably modified for use in user-space code.
 */
#include <linux/list.h>

/* These are all Portals3 include files.
 */
#include <p3utils.h>

#include <p3api/types.h>
#include <p3api/debug.h>

#include <p3/lock.h>
#include <p3/handle.h>
#include <p3/process.h>
#include <p3/forward.h>
#include <p3/errno.h>
#include <p3/debug.h>

#include <p3lib/types.h>
#include <p3lib/p3lib.h>
#include <p3lib/nal.h>
#include <p3lib/p3lib_support.h>
#include <p3lib/dispatch.h>


#define PTL_DISPATCH_ENTRY(fcn_id,fcn) {lib_##fcn, #fcn}

dispatch_table_t dispatch_table[] =
{
	PTL_LIB_DISPATCH,
	{NULL, ""}
};

#undef PTL_DISPATCH_ENTRY

void
lib_dispatch(p3_libreq_t *req, p3_libres_t *res)
{
	p3_process_t *pp;
	lib_ni_t *ni = NULL;
	unsigned i, cid = req->call_index;

	if (cid > LIB_MAX_DISPATCH || !dispatch_table[cid].fun)
		return;

	/* Normally every valid request must have a current Portals
	 * process, and a valid network interface.  The exceptions are:
	 *
	 * PTL_LIB_INIT: has neither, and creates the current Portals process.
	 * PTL_NI_INIT: needs a current Portals process, and creates an NI.
	 * PTL_LIB_FINI: only needs a current Portals process, as all NIs
	 *	may already have been released.
	 * PTL_NI_DEBUG: we specifically allow this command to not need
	 *	an NI, in which case the debug value is set for all future NIs
	 */
	if ((pp = p3lib_cur_process()) &&
	    ((i = PTL_NI_INDEX(req->handle)) &&
	     i < PTL_MAX_INTERFACES && (ni = pp->ni[i]))) {
		
		p3_lock(&ni->obj_alloc);
		if (!TST_OBJ(ni, OBJ_INUSE))
			res->status = PTL_HANDLE_INVALID;
		p3_unlock(&ni->obj_alloc);
		if (res->status == PTL_HANDLE_INVALID)
			return;
	}
	else if (!pp) {
#if 0
		if (cid == P3LIB_LIB_FINI) {
			p3_print("lib_dispatch: detected fork\n\n");
		}
		else 
#endif
		if (cid != P3LIB_LIB_INIT) {
			res->status = PTL_NO_INIT;
			return;
		}
	}
	else if (!ni) {
		if ( cid != P3LIB_LIB_FINI && 
		     cid != P3LIB_NI_INIT && 
		     cid != P3LIB_NI_DEBUG) {
			res->status = PTL_HANDLE_INVALID;
			return;
		}
	}
	if (DEBUG_P3((ni ? ni->debug : 0), PTL_DBG_API))
		p3_print("lib_dispatch:"FMT_NIDPID": API call %s (index %d)\n",
			 (ni ? ni->nid : PTL_NID_ANY),
			 (ni ? ni->pid : PTL_PID_ANY), 
			 dispatch_name(cid), cid);

	dispatch_table[cid].fun(ni, req, res);
}

char *dispatch_name(int cid)
{
	return dispatch_table[cid].name;
}
