/*
 * This Cplant(TM) source code is part of the Portals3 Reference
 * Implementation.
 *
 * This Cplant(TM) source code is the property of Sandia National
 * Laboratories.
 *
 * This Cplant(TM) source code is copyrighted by Sandia National
 * Laboratories.
 *
 * The redistribution of this Cplant(TM) source code is subject to the
 * terms of version 2 of the GNU General Public License.
 * (See COPYING, or http://www.gnu.org/licenses/lgpl.html.)
 *
 * Cplant(TM) Copyright 1998-2006 Sandia Corporation. 
 *
 * Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
 * license for use of this work by or on behalf of the US Government.
 * Export of this program may require a license from the United States
 * Government.
 */


/* Portals3 is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License,
 * as published by the Free Software Foundation.
 *
 * Portals3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Portals3; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Questions or comments about this library should be sent to:
 *
 * Jim Schutt
 * Sandia National Laboratories, New Mexico
 * P.O. Box 5800
 * Albuquerque, NM 87185-0806
 *
 * jaschut@sandia.gov
 *
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>

/* If compiling for user-space (and maybe NIC-space), these need to be
 * the Portals3 versions of the Linux kernel include files, which
 * have been suitably modified for use in user-space code.
 */
#include <linux/list.h>

/* These are all Portals3 include files.
 */
#include <p3-config.h>

#include <p3api/types.h>

#include <p3/lock.h>
#include <p3/handle.h>
#include <p3/process.h>
#include <p3/forward.h>
#include <p3/errno.h>

static int lib_fd = -1;

void p3_forward(void *request, size_t reqlen, void *result, size_t reslen)
{
	p3_forward_t fw = {request, result, reqlen, reslen};
	ssize_t n;

#ifndef NALS_NEED_PROGRESS_SUPPORT
	if (((p3_libreq_t *)request)->call_index == P3LIB_PROGRESS) {
		((p3_libres_t *)result)->status = PTL_OK;
		return;
	}
#endif

	if (lib_fd < 0) {
		lib_fd = open("/dev/p3lib", O_WRONLY);
		if (lib_fd < 0) {
			perror("p3_forward: open /dev/p3lib");
			return;
		}
	}
	n = write(lib_fd, &fw, sizeof(fw));
	if (n < 0) {
		perror("p3_forward: write request block");
		return;
	}
	if (n != sizeof(fw))
		fprintf(stderr, "p3_forward: short write on request block\n");
}

void p3_forward_shutdown(void)
{
	if (lib_fd >= 0) {
		close(lib_fd);
		lib_fd = -1;
	}
}
